#!/bin/bash
#
# g64_masscopier.sh
#
# written on 30.01.2021  by GI-Joe
#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# This script facilitates the mass copying of floppy disks from a 1541/1571-diskdrive
# to a Linux machine with a XUM-adapter. The XUM-driver, opencbm and nibtools must be
# installed on the system.
# This script will create 4 files for each diskside in a folder ${FILEPREFIX}${DISKCOUNTERSTRING}:
# xxxxxyyyzz.nib, xxxxxyyyzz.g64, xxxxxyyyzz.d64, xxxxxyyyzz.log
# +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# Parameter 1 : devicenumber (8|9|10|11)
# Parameter 2 : destination directory where all D64 files are copied (directory is created if does not exist)
# Parameter 3 : prefix of the file name
# Parameter 4 : start number of the disk counter
#
#================================================================================================================================================================
#================================================================================================================================================================
#==== nibread-options - just change it at will ==================================================================================================================
#================================================================================================================================================================
XUMADAPTER="-@xum1541:0"	#-@x: Use OpenCBM device 'x' (xa1541, xum1541:0, xum1541:1, etc.)
RETRYONERRORS="-e2"			#-e[n]: Retry reading tracks with errors [n] times
STARTTRACK="-S1"			#-S[n]: Override starting track
ENDTRACK="-E35"				#-E[n]: Override ending track
MATCHTRACKGAP=""			#-G[n]: Match track gap by [n] number of bytes (advanced users only)
USEPARALLELTRANSFER=""		#-P: Use parallel transfer instead of SRQ (1571 only)
DISABLEREADINGKILLERTRACKS="-k"	#-k: Disable reading of 'killer' tracks
FORCEDEFAULTDENSITYS=""		#-d: Force default densities
ENABLETRACKMATCHING=""		#-v: Enable track matching (crude read verify)
INTERACTIVEIMAGINGMODE=""	#-I: Interactive imaging mode
VERBOSE=""					#-V: Verbose (output more detailed track data)
READHALFTRACKS=""			#-h: Read halftracks
EXTENDEDPARALLELPORTTESTS=""	#-t: Extended parallel port tests
USEINDESHOLESENSOR=""		#-j: Use Index Hole Sensor  (1541/1571 SC+ compatible IHS)
TRACKALIGNMENTREPORT=""		#-x: Track Alignment Report (1541/1571 SC+ compatible IHS)
DEEPBITRATEANALYSE=""		#-y: Deep Bitrate Analysis  (1541/1571 SC+ compatible IHS)
TESTINDEXHOLESENSOR=""		#-z: Test Index Hole Sensor (1541/1571 SC+ compatible IHS)
#
#================================================================================================================================================================
#================================================================================================================================================================
#==== variables / constants  - just change it at will ===========================================================================================================
#================================================================================================================================================================
TERMINAL="/usr/bin/xfce4-terminal"	# if you use another terminal, you should edit it here ;)
SIDEA="S1"					# this text will be added in fileprefix for diskside A
SIDEB="S2"					# this text will be added in fileprefix for diskside B
EXTENDED_TRACK_TEST="YES"	#  YES|NO - tests each disc before copying whether the tracks 36-41 are readable and if so, these tracks are copied too.
TRACKTESTFILE="/tmp/tracktest.nib"		# temporary filename of the extra-tracks-test - the suffix MUST be ".nib" !!
TRACKTESTLOGFILE="/tmp/tracktest.log"	# temporary filename of the extra-tracks-test - logfile
G64FILESUFFIX="g64"
NIBFILESUFFIX="nib"
FILESUFFIX="d64"
TESTSTARTTRACK="-S36" 	# option for nibread (test whether the extra tracks are readable, -Sxx is the starttrack of extratrack-testing)
TESTENDTRACK="-E40"		# option for nibread (test whether the extra tracks are readable, -Exx is the endtrack of extratrack-testing)
PARAMETERNUMBER="$#"	# don't change it please !
FILEPREFIX="$3"			# dont change it please !
DEVICENUMBER=`echo "$1" | sed -e 's/^0*//'`		# delete the leading zeros
DESTINATIONDIR=`echo "$2" | sed {"s/\/$//g"}` 	# delete the last slash if exist
DISKCOUNTERSTART=`echo 10#$4 | sed -e 's/^0*//'` # delete the leading zeros
#
#================================================================================================================================================================
#================================================================================================================================================================
#==== functions =================================================================================================================================================
#================================================================================================================================================================
function PrintUsageText {
	echo ""
	echo "at least 1 parameter is missing or incorrect !"
	echo "usage:"
	echo './g64_masscopier.sh [devicenumber] [destinationdir] [destination-file-prefix] [startnumber-diskcounter]'
	echo 'Parameter 1 : devicenumber (8|9|10|11)'
	echo 'Parameter 2 : destination directory where all D64 files are copied (directory is created if does not exist)'
	echo 'Parameter 3 : prefix of the destination filename'
	echo 'Parameter 4 : initial start number of the disk counter'
	echo ""
	echo ""
	exit 1
}
#
function SetExtendedTrackParameters {			# test whether the extra tracks are readable and re-set the option for the endtrack
	[ -f ${TRACKTESTFILE} ] && rm ${TRACKTESTFILE}
	cbmctrl reset
	if cbmctrl detect | grep 1571 >/dev/null 2>&1
		then
			nibread -D$DEVICENUMBER -s $TESTSTARTTRACK $TESTENDTRACK ${TRACKTESTFILE} | tee ${TRACKTESTLOGFILE} # 2>>${TRACKTESTLOGFILE}
		else
			nibread -D$DEVICENUMBER $TESTSTARTTRACK $TESTENDTRACK ${TRACKTESTFILE} | tee ${TRACKTESTLOGFILE} # 2>>${TRACKTESTLOGFILE}
	fi
	sleep 1
	if grep '41.0' ${TRACKTESTLOGFILE} | grep '\[Unformatted Track\]' || grep '41.0' ${TRACKTESTLOGFILE} | grep 'NOSYNC!' || grep '41.0' ${TRACKTESTLOGFILE} | grep '\[NDOS\]' || grep '41.0' ${TRACKTESTLOGFILE} | grep '\[Killer Track\]'
		then
			ENDTRACK="-E40"
		elif grep '41.0' ${TRACKTESTLOGFILE} >/dev/null 2>&1 ; then
			ENDTRACK="-E41" && return
	fi
	if grep '40.0' ${TRACKTESTLOGFILE} | grep '\[Unformatted Track\]' || grep '40.0' ${TRACKTESTLOGFILE} | grep 'NOSYNC!' || grep '40.0' ${TRACKTESTLOGFILE} | grep '\[NDOS\]' || grep '40.0' ${TRACKTESTLOGFILE} | grep '\[Killer Track\]'
		then
			ENDTRACK="-E39"
		elif grep '40.0' ${TRACKTESTLOGFILE} >/dev/null 2>&1 ; then
			ENDTRACK="-E40" && return
	fi
	if grep '39.0' ${TRACKTESTLOGFILE} | grep '\[Unformatted Track\]' || grep '39.0' ${TRACKTESTLOGFILE} | grep 'NOSYNC!' || grep '39.0' ${TRACKTESTLOGFILE} | grep '\[NDOS\]' || grep '39.0' ${TRACKTESTLOGFILE} | grep '\[Killer Track\]'
		then
			ENDTRACK="-E38"
		elif grep '39.0' ${TRACKTESTLOGFILE} >/dev/null 2>&1 ; then
			ENDTRACK="-E39" && return
	fi
	if grep '38.0' ${TRACKTESTLOGFILE} | grep '\[Unformatted Track\]' || grep '38.0' ${TRACKTESTLOGFILE} | grep 'NOSYNC!' || grep '38.0' ${TRACKTESTLOGFILE} | grep '\[NDOS\]' || grep '38.0' ${TRACKTESTLOGFILE} | grep '\[Killer Track\]'
		then
			ENDTRACK="-E37"
		elif grep '38.0' ${TRACKTESTLOGFILE} >/dev/null 2>&1 ; then
			ENDTRACK="-E38" && return
	fi
	if grep '37.0' ${TRACKTESTLOGFILE} | grep '\[Unformatted Track\]' || grep '37.0' ${TRACKTESTLOGFILE} | grep 'NOSYNC!' || grep '37.0' ${TRACKTESTLOGFILE} | grep '\[NDOS\]' || grep '37.0' ${TRACKTESTLOGFILE} | grep '\[Killer Track\]'
		then
			ENDTRACK="-E36"
		elif grep '37.0' ${TRACKTESTLOGFILE} >/dev/null 2>&1 ; then
			ENDTRACK="-E37" && return
	fi
	if grep '36.0' ${TRACKTESTLOGFILE} | grep '\[Unformatted Track\]' || grep '36.0' ${TRACKTESTLOGFILE} | grep 'NOSYNC!' || grep '36.0' ${TRACKTESTLOGFILE} | grep '\[NDOS\]' || grep '36.0' ${TRACKTESTLOGFILE} | grep '\[Killer Track\]'
		then
			ENDTRACK="-E35"
		elif grep '36.0' ${TRACKTESTLOGFILE} >/dev/null 2>&1 ; then
			ENDTRACK="-E36" && return
	fi
}
#
function PrepareForCopySideA {
	[ "$EXTENDED_TRACK_TEST" = "YES" ] && echo -e '\033[1;37mtesting for extra tracks ...\033[0m' && SetExtendedTrackParameters
	[ -f ${TRACKTESTLOGFILE} ] && rm ${TRACKTESTLOGFILE}
	[ -d ${DESTINATIONDIR}/${FILEPREFIX}_${DISKCOUNTERSTRING} ] && rm -R ${DESTINATIONDIR}/${FILEPREFIX}_${DISKCOUNTERSTRING}
	mkdir ${DESTINATIONDIR}/${FILEPREFIX}_${DISKCOUNTERSTRING}
	echo "nibread $XUMADAPTER $DRIVETYPE $STARTTRACK $ENDTRACK $TRANSFERMODE $WARPMODE $RETRYCOUNT $BAMONLY $BAMSAVE $ERRORMAP $TWOSIDED $NOPROGRESS $DEVICENUMBER ${DESTINATIONDIR}/${FILEPREFIX}_${DISKCOUNTERSTRING}/${FILEPREFIX}_${DISKCOUNTERSTRING}-${SIDEA}.${NIBFILESUFFIX}"
	cbmctrl reset
}
#
function PrepareForCopySideB {
	[ "$EXTENDED_TRACK_TEST" = "YES" ] && echo -e '\033[1;37mtesting for extra tracks ...\033[0m' && SetExtendedTrackParameters
	[ -f ${TRACKTESTLOGFILE} ] && rm ${TRACKTESTLOGFILE}
	echo "nibread $XUMADAPTER $DRIVETYPE $STARTTRACK $ENDTRACK $TRANSFERMODE $WARPMODE $RETRYCOUNT $BAMONLY $BAMSAVE $ERRORMAP $TWOSIDED $NOPROGRESS $DEVICENUMBER ${DESTINATIONDIR}/${FILEPREFIX}_${DISKCOUNTERSTRING}/${FILEPREFIX}_${DISKCOUNTERSTRING}-${SIDEB}.${NIBFILESUFFIX}"
	cbmctrl reset
}
#
#================================================================================================================================================================
#================================================================================================================================================================
#================================================================================================================================================================
#==== Scriptcode ================================================================================================================================================
#================================================================================================================================================================
[ ! -d ${DESTINATIONDIR} ] && mkdir -p ${DESTINATIONDIR}
#
if [ $PARAMETERNUMBER -eq 4 ] # check whether the number of commandline-parameters is correct after script start
	then
		sleep 0
	else
		echo "ERROR: PARAMETERNUMBER is bad !"
		PrintUsageText
fi
#
shopt -s extglob
if [[ -n "$DEVICENUMBER" && -z "${DEVICENUMBER##?([0-9])}" ]] # is parameter 1 an integer ?
	then
		typeset -i DEVICENUMBER
	else
		echo "ERROR: parameter 1 is not an integer"
		PrintUsageText
fi
#
if [[ $DEVICENUMBER -ge 8 && $DEVICENUMBER -le 11 ]] # is the value of parameter 1 between 8 and 11 ?
	then
		typeset -i DEVICENUMBER
	else
		echo "ERROR: parameter 1 is not between 8 and 11"
		PrintUsageText
fi
#
if [[ "$DISKCOUNTERSTART" =~ ^[0-9]+$ ]] # is parameter 4 an integer ?
	then
		echo "ERROR: parameter 4 is not an integer"
		PrintUsageText
	else
		typeset -i DISKCOUNTERSTART
		DISKCOUNTERSTART=$DISKCOUNTERSTART+0
fi
#
if cbmctrl detect >/dev/null 2>&1 	# is the xum1541-adapter connected ?
	then
		cbmctrl reset
	else 
		echo "ERROR: no xum1541 device found - exit !"
	exit 1
fi
#
#
#================================================================================================================================================================
#================================================================================================================================================================
#==== Mainloop ==================================================================================================================================================
#================================================================================================================================================================
DISKCOUNTERSTRING=`printf "%.3i\n" $DISKCOUNTERSTART` # change counter number to 3 digits 
echo ""
echo ""
echo ""
cbmctrl status $DEVICENUMBER
echo ""
echo -e "\033[1;32mInsert Disc for creating:  ${DESTINATIONDIR}/${FILEPREFIX}_${DISKCOUNTERSTRING}-${SIDEA}.${NIBFILESUFFIX}\033[0m"
echo -e "\033[1;36mpress RETURN to start or CTRL-C for cancel\033[0m" && read a
while true ; do
	DISKCOUNTERSTRING=`printf "%.3i\n" $DISKCOUNTERSTART` # change counter number to 3 digits 
	PrepareForCopySideA
 	echo "nibread $XUMADAPTER $RETRYONERRORS $STARTTRACK $ENDTRACK $MATCHTRACKGAP $USEPARALLELTRANSFER $DISABLEREADINGKILLERTRACKS $FORCEDEFAULTDENSITYS $ENABLETRACKMATCHING $INTERACTIVEIMAGINGMODE $VERBOSE $READHALFTRACKS $EXTENDEDPARALLELPORTTESTS $EXTENDEDPARALLELPORTTESTS $USEINDESHOLESENSOR $TRACKALIGNMENTREPORT $DEEPBITRATEANALYSE $TESTINDEXHOLESENSOR -D$DEVICENUMBER ${DESTINATIONDIR}/${FILEPREFIX}_${DISKCOUNTERSTRING}/${FILEPREFIX}_${DISKCOUNTERSTRING}-${SIDEB}.${NIBFILESUFFIX}"

	$TERMINAL --hide-menubar --hide-toolbar --hide-scrollbar --geometry=50x54 --title="COPY IN PROGRESS" --command="nibread $XUMADAPTER $RETRYONERRORS $STARTTRACK $ENDTRACK $MATCHTRACKGAP $USEPARALLELTRANSFER $DISABLEREADINGKILLERTRACKS $FORCEDEFAULTDENSITYS $ENABLETRACKMATCHING $INTERACTIVEIMAGINGMODE $VERBOSE $READHALFTRACKS $EXTENDEDPARALLELPORTTESTS $EXTENDEDPARALLELPORTTESTS $USEINDESHOLESENSOR $TRACKALIGNMENTREPORT $DEEPBITRATEANALYSE $TESTINDEXHOLESENSOR -D$DEVICENUMBER ${DESTINATIONDIR}/${FILEPREFIX}_${DISKCOUNTERSTRING}/${FILEPREFIX}_${DISKCOUNTERSTRING}-${SIDEA}.${NIBFILESUFFIX}"
	while ps a | grep "nibread" | grep -v grep >/dev/null ; do sleep 1 ; done
	nibconv ${DESTINATIONDIR}/${FILEPREFIX}_${DISKCOUNTERSTRING}/${FILEPREFIX}_${DISKCOUNTERSTRING}-${SIDEA}.${NIBFILESUFFIX} ${DESTINATIONDIR}/${FILEPREFIX}_${DISKCOUNTERSTRING}/${FILEPREFIX}_${DISKCOUNTERSTRING}-${SIDEA}.${FILESUFFIX}
	nibconv ${DESTINATIONDIR}/${FILEPREFIX}_${DISKCOUNTERSTRING}/${FILEPREFIX}_${DISKCOUNTERSTRING}-${SIDEA}.${NIBFILESUFFIX} ${DESTINATIONDIR}/${FILEPREFIX}_${DISKCOUNTERSTRING}/${FILEPREFIX}_${DISKCOUNTERSTRING}-${SIDEA}.${G64FILESUFFIX}
	echo ""
	echo -e "\033[1;33mplease insert the same Disk - SideB ... - \033[1;36mCTRL-C for CANCEL\033[0m" && cbmctrl change ${DEVICENUMBER}
	PrepareForCopySideB
	$TERMINAL --hide-menubar --hide-toolbar --hide-scrollbar --geometry=50x54 --title="COPY IN PROGRESS" --command="nibread $XUMADAPTER $RETRYONERRORS $STARTTRACK $ENDTRACK $MATCHTRACKGAP $USEPARALLELTRANSFER $DISABLEREADINGKILLERTRACKS $FORCEDEFAULTDENSITYS $ENABLETRACKMATCHING $INTERACTIVEIMAGINGMODE $VERBOSE $READHALFTRACKS $EXTENDEDPARALLELPORTTESTS $EXTENDEDPARALLELPORTTESTS $USEINDESHOLESENSOR $TRACKALIGNMENTREPORT $DEEPBITRATEANALYSE $TESTINDEXHOLESENSOR -D$DEVICENUMBER ${DESTINATIONDIR}/${FILEPREFIX}_${DISKCOUNTERSTRING}/${FILEPREFIX}_${DISKCOUNTERSTRING}-${SIDEB}.${NIBFILESUFFIX}"
	while ps a | grep "nibread" | grep -v grep >/dev/null ; do sleep 1 ; done
	nibconv ${DESTINATIONDIR}/${FILEPREFIX}_${DISKCOUNTERSTRING}/${FILEPREFIX}_${DISKCOUNTERSTRING}-${SIDEB}.${NIBFILESUFFIX} ${DESTINATIONDIR}/${FILEPREFIX}_${DISKCOUNTERSTRING}/${FILEPREFIX}_${DISKCOUNTERSTRING}-${SIDEB}.${FILESUFFIX}
	nibconv ${DESTINATIONDIR}/${FILEPREFIX}_${DISKCOUNTERSTRING}/${FILEPREFIX}_${DISKCOUNTERSTRING}-${SIDEB}.${NIBFILESUFFIX} ${DESTINATIONDIR}/${FILEPREFIX}_${DISKCOUNTERSTRING}/${FILEPREFIX}_${DISKCOUNTERSTRING}-${SIDEB}.${G64FILESUFFIX}
	echo ""
	echo -e "\033[1;32mplease insert next Disk - SideA ... - \033[1;36mCTRL-C for CANCEL\033[0m" && cbmctrl change ${DEVICENUMBER}
	DISKCOUNTERSTART=$DISKCOUNTERSTART+1
done
#
#
exit

